%% Transfer Function
% This file evaluates a first-order transfer function at s=j*omega
% and plots the resulting number in the complex plane.


%% Transfer Function
b0 = 2;
a0 = 4;
G = tf(b0,[1 a0]);


%% Evaluate G at a single frequency, w
% Note: The function evalfr(G,s) evaluates the transfer function
% at the complex value s.  For the first-order system above,
%  evalfr(G,jw) is equivalent to b0/(j*w+a0).
w = 3;                      % Frequency, rad/sec
Gw = evalfr(G,j*w);         % Transfer function at s=wjw

% Cartesian form (real/imaginary)
ReGw = real(Gw)
ImGw = imag(Gw)

% Polar form (magnitude/phase)
MagGw = abs(Gw)
PhGw = angle(Gw)

%% Plot G evaluated at a single frequency

figure(1)
clf;
plot(ReGw,ImGw,'rx','MarkerSize',14);
hold on;
xlabel('Real');
ylabel('Imag');

% Lines for axes and from origin to roots
%ax=[-0.25 0.6 -0.3 0.1];
ax=[-0.3 0.6 -0.4 0.15];
axis(ax);
ph(1)=plot(0.97*[0 ReGw],0.97*[0,ImGw],'k:');
ph(2)=plot([0 0],ax(3:4),'k');
ph(3)=plot(ax(1:2),[0 0],'k');
set(gca, 'LooseInset', get(gca, 'TightInset'));

% Text Labels
th(1) = text(0.65*ReGw,0.6*ImGw,'|G(3j)|','FontSize',18);
ph(4)=plot([ReGw ReGw],0.02*[-1 1],'k');
th(2) = text(ReGw,0.05,'Re\{G(3j)\}','FontSize',18,...
    'HorizontalAlignment','Center');
ph(5)=plot(0.02*[-1 1],[ImGw ImGw],'k');
th(3) = text(-0.3,ImGw,'Im\{G(3j)\}','FontSize',18);
th(4) = text(0.11,-0.05,'\angle G(3j)','FontSize',18);

% Draw arc arrow
theta = linspace(PhGw+0.05,-0.05,25);
xx = 0.13*cos(theta);
yy = 0.13*sin(theta);
ph(6)= plot(xx,yy,'k');

hold off;

%% Evaluate G on a grid of frequencies and plot results
wgrid = [0 1 2 4 6 10 20 inf]';
Ngrid = numel(wgrid);
hold on;
Ggrid = zeros(Ngrid,1);
for i=1:Ngrid
    wi = wgrid(i);
    Ggrid(i) = evalfr(G,j*wi);
end
plot(real(Ggrid),imag(Ggrid),'r.','MarkerSize',12);
hold off

[ wgrid real(Ggrid) imag(Ggrid) abs(Ggrid) angle(Ggrid)]


%% Save Plot

if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    set(ph,'LineWidth',1.5)
    for i=1:numel(th)
        set(th(i),'String',['$' get(th(i),'String') '$']);
    end
    
    % 1) To shrink font add "\small" after "\begin{tikzpicture}"
    % 2) To add arrowheads: \addplot[<->,...   
    warning('Not Saving File')
%     matlab2tikz('TFNotation.tikz','width','\figurewidth',...
%         'parseStrings',false,'extraAxisOptions',...
%         'ylabel near ticks, xlabel near ticks');
end

    