%% Bode Plots for Integrators and Differentiators
% This file generates Bode plots and frequency responses for systems
% that integrate or differentiate the input signal.

%% Integrator
% The dynamics of an integrator are: d(y)/dt = u 
% In other words, the output is simply the integral of the input.
% The corresponding transfer function is G(s) = 1/s.

% Create transfer function and state-space form for an integrator
G = tf(1,[1 0]);
Gss = ss(0,1,1,0);

% Create Bode Plot
[Gmag,Gph_deg,w] = bode(G);
Gmag = Gmag(:);
Gph_deg = Gph_deg(:);
Gmag_dB = 20*log10(Gmag);

figure(1); clf;
subplot(2,1,1)
semilogx(w,Gmag_dB,'b')
ylabel('Magnitude (dB)');
title('Bode Diagram');
grid on;
set(gca,'LineWidth',1,'Ytick',[-20 -6 0 6 20]);
set(gca,'Xtick',[0.1 0.5 1 2 10]);

subplot(2,1,2)
semilogx(w,Gph_deg,'b')
ylim([-135 0]);
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
set(gca,'LineWidth',1,'Ytick',[-135 -90 -45 0]);
set(gca,'Xtick',[0.1 0.5 1 2 10]);
grid on;

% Generate frequency responses at several frequencies
w = [0.5 2];         % Frequencies to simulate, rad/sec
Nw = numel(w);
figure(2); clf;
for i=1:Nw
    % Evaluate TF at specified frequency
    Gw = evalfr(Gss,j*w(i));
    Gmag = abs(Gw);
    Gmag_db = 20*log10( Gmag );
    Gph_deg = (180/pi)*angle(Gw);    
    
    % Plot circle on Bode plot
    figure(1);
    subplot(2,1,1); 
    hold on;
    semilogx(w(i),Gmag_db,'ro');
    hold off;
    subplot(2,1,2); 
    hold on;
    semilogx(w(i),Gph_deg,'ro');
    hold off;
    
    % Simulate system
    y0 = -Gmag;
    Tf = 6*pi;
    t = linspace(0,Tf,50*Tf);
    u = sin(w(i)*t);
    [y,t] = lsim(Gss,u,t,y0);
    
    % Plot results
    figure(2);
    subplot(2,1,i);
    plot(t,u,'b-.',t,y,'r')
    xlim([0 Tf]);
    xlabel('Time, t (sec)');
    ylabel('Output, y');
    grid on;    
    set(gca, 'LooseInset', get(gca, 'TightInset'));

    % Set ticks
    if i==1
        legend('u, \omega=0.5','y','Location','SouthEast');
        set(gca,'Ytick',[-2 -1 0 1 2]);
    else
        legend('u, \omega=2','y','Location','SouthEast');
        set(gca,'Ytick',[-1 -0.5 0 0.5 1]);
    end
    set(gca,'LineWidth',1)    
    set(gca,'Xtick',0:pi:Tf,'XtickLabel',...
        {'0', '\pi', '2\pi', '3\pi','4\pi', '5\pi', '6\pi'});
end

% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'IntBode')
    
    figure(2)
    garyfyFigure    
    myprint(2,'IntResp')
end

%% Differentiator
% The dynamics of an differentiator are: y = d(u)/dt
% In other words, the output is simply the derivative of the input.
% The corresponding transfer function is G(s) = s/1 = s.

% Create transfer function for a differentiator
G = tf([1 0],1);

% Create Bode Plot
[Gmag,Gph_deg,w] = bode(G);
Gmag = Gmag(:);
Gph_deg = Gph_deg(:);
Gmag_dB = 20*log10(Gmag);

figure(3); clf;
subplot(2,1,1)
semilogx(w,Gmag_dB,'b')
ylabel('Magnitude (dB)');
title('Bode Diagram');
grid on;
set(gca,'LineWidth',1,'Ytick',[-20 -6 0 6 20]);
set(gca,'Xtick',[0.1 0.5 1 2 10]);

subplot(2,1,2)
semilogx(w,Gph_deg,'b')
ylim([0 135]);
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
set(gca,'LineWidth',1,'Ytick',[0 45 90 135]);
set(gca,'Xtick',[0.1 0.5 1 2 10]);
grid on;

% Generate frequency responses at several frequencies
w = [0.5 2];         % Frequencies to simulate, rad/sec
Nw = numel(w);
figure(4); clf;
for i=1:Nw
    % Evaluate TF at specified frequency
    Gw = evalfr(G,j*w(i));
    Gmag = abs(Gw);
    Gmag_db = 20*log10( Gmag );
    Gph_deg = (180/pi)*angle(Gw);    
    
    % Plot circle on Bode plot
    figure(3);
    subplot(2,1,1); 
    hold on;
    semilogx(w(i),Gmag_db,'ro');
    hold off;
    subplot(2,1,2); 
    hold on;
    semilogx(w(i),Gph_deg,'ro');
    hold off;
    
    % Simulate system
    Tf = 6*pi;
    t = linspace(0,Tf,50*Tf);
    u = sin(w(i)*t);
    y = w(i)*cos(w(i)*t);
%    [y,t] = lsim(G,u,t);
    
    % Plot results
    figure(4);
    subplot(2,1,i);
    plot(t,u,'b-.',t,y,'r')
    xlim([0 Tf]);
    xlabel('Time, t (sec)');
    ylabel('Output, y');
    grid on;
    set(gca, 'LooseInset', get(gca, 'TightInset'));

    % Set ticks
    if i==1
        legend('u, \omega=0.5','y','Location','SouthEast');
        set(gca,'Ytick',[-1 -0.5 0 0.5 1]);
    else
        legend('u, \omega=2','y','Location','SouthEast');
        set(gca,'Ytick',[-2 -1 0 1 2]);
    end
    set(gca,'Xtick',0:pi:Tf,'XtickLabel',...
        {'0', '\pi', '2\pi', '3\pi','4\pi', '5\pi', '6\pi'});
    set(gca,'LineWidth',1)    
end


% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(3)
    garyfyFigure
    myprint(3,'DifBode')
    
    figure(4)
    garyfyFigure    
    myprint(4,'DifResp')
end


