%% Bode Plot: First-order pole
% This file generates a Bode plot for a first order system using the
% Matlab command "bode".

%% Transfer Function
b0 = 2;
a0 = 4;
G = tf(b0,[1 a0]);


%% Exact Bode Plot

% Use Bode command to compute exact magnitude and phase data. 
w = sort([a0 a0/10 10*a0 logspace(-2,3,200)]);
[Gmag,Gph_deg,w] = bode(G,w);
Gmag = Gmag(:);
Gph_deg = Gph_deg(:);
Gmag_dB = 20*log10(Gmag);
G0 = b0/a0;
G0dB = 20*log10(b0/a0);

figure(1); clf;

% Magnitude Plot
subplot(2,1,1);
semilogx(w/a0,Gmag_dB,'b');
ylabel('Magnitude (dB)');
title('Bode Diagram');
grid on;
axis([1/100 100 -52 0]);
set(gca,'LineWidth',2);
set(gca,'YTick',[-46 -26 -9 -6],'YTickLabel',{'-46','-26','',''});
th(1) = text(0.61/100,-3,'-6','FontSize',14);
th(2) = text(0.61/100,-11,'-9','FontSize',14);
set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
    'XtickLabel',{'0.04', '0.4', '4', '40', '400'});
hold off;

% Phase Plot
subplot(2,1,2);
semilogx(w/a0,Gph_deg,'b');
axis([1/100 100 -90 0]);
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
grid on;
set(gca,'LineWidth',2);
set(gca,'YTick',[-90 -45 0]);
set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
    'XtickLabel',{'0.04', '0.4', '4', '40', '400'});
hold off;

% Save Plot
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure    
    myprint(1,'BodeFOPole1')
end


%% Straight-line (asymptotic) Bode approximation

% Normalize frequency by corner frequency
w = w/a0;

% Frequencies for straight-line approximations
wlow = w( w<= 1/10);
wmid1 = w( w>= 1/10 & w<=1 );
wmid2 = w( w>= 1 & w<=10 );
whigh = w( w>= 10);
Nw = [numel(wlow) numel(wmid1) numel(wmid2) numel(whigh)];

figure(2);
clf;

% Magnitude Plot
subplot(2,1,1);
ph(1) = semilogx(wlow, G0dB*ones(Nw(1),1),'r'); 
hold on;
ph(2) = semilogx(wmid1, G0dB*ones(Nw(2),1),'r');
ph(3) = semilogx(whigh, 20*log10(G0./whigh),'r');
ph(4) = semilogx(wmid2, 20*log10(G0./wmid2),'r');
ph(5) = semilogx(w,Gmag_dB,'b:');
ylabel('Magnitude (dB)');
title('Bode Diagram');
grid on;
axis([1/100 100 G0dB-46 G0dB+6]);
set(gca,'LineWidth',2);
set(gca,'YTick',G0dB-[40 20 3 0],...
    'YtickLabel',{'','','   ',''});
th(1) = text(0.71/100,-3,'g','FontSize',14);
th(2) = text(1.2/100,-11,'g-3','FontSize',14);
th(3) = text(1.2/100,-26,'g-20','FontSize',14,'BackgroundColor','w');
th(4) = text(1.2/100,-44,'g-40','FontSize',14,'BackgroundColor','w');
set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
    'XtickLabel',{'a_0/100', 'a_0/10', 'a_0', '10a_0', '100a_0'});
hold off;

% Phase Plot
subplot(2,1,2);
ph(6) = semilogx(wlow, 0*ones(Nw(1),1),'r');
hold on;
wmid = [wmid1; wmid2];
fac = log10(wmid/wmid(1)) / log10(wmid(end)/wmid(1));
ph(7) = semilogx(wmid, -90*fac,'r');
ph(8) = semilogx(whigh, -90*ones(Nw(4),1),'r');
ph(9)=semilogx(w,Gph_deg,'b:');
axis([1/100 100 -90 0]);
legend(ph([6 9]),'Approx','Actual','Location','NorthEast');
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
grid on;
set(gca,'LineWidth',2);
set(gca,'YTick',[-90 -45 0]);
set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
    'XtickLabel',{'a_0/100', 'a_0/10', 'a_0', '10a_0', '100a_0'});
hold off;

% Save Plot
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(2)
    garyfyFigure    
    set(ph([1:4 6:8]),'LineWidth',2)
    myprint(2,'BodeFOPole2')
end
