%% Second Order Resonance
% This file generates a Bode plot for an underdamped second order
% system. Frequency responses are also computed for the system to
% demonstrate the response at resonance.

%% Second Order Underdamped System

% Create transfer function
b0 = 16;
wn = 4;
z = 0.05;
G = tf(b0,[1 2*z*wn wn^2]);

% Create Bode Plot
w = sort([wn wn/10 10*wn logspace(-2,3,200)]);
[Gmag,Gph_deg,w] = bode(G,w);
Gmag = Gmag(:);
Gph_deg = Gph_deg(:);
Gmag_dB = 20*log10(Gmag);
G0 = b0/wn^2;
G0dB = 20*log10(G0);

% Magnitude Plot
figure(1);
clf
subplot(2,1,1);
semilogx(w/wn,Gmag_dB,'b');
ylabel('Magnitude (dB)');
title('Bode Diagram');
grid on;
axis([1/100 100 -90 28]);
set(gca,'LineWidth',2);
set(gca,'YTick',[-80 -40 -9.5 0 20],...
        'YTickLabel',{'-80','-40','','0','20'});
text(0.44/100,-14,'-9.5','FontSize',14);
set(gca,'Xtick',[1/100 1/10 1 2 10 100],'XMinorGrid','off', ...
    'XtickLabel',{'0.04', '0.4', '4', '8', '40', '400'});

% Phase Plot
subplot(2,1,2);
semilogx(w/wn,Gph_deg,'b');
axis([1/100 100 -180 0]);
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
grid on;
set(gca,'LineWidth',2);
set(gca,'YTick',[-180 -135 -90 -45 0]);
set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
    'XtickLabel',{'0.04', '0.4', '4', '40', '400'});


% Generate frequency responses at several frequencies
w = [8 4];         % Frequencies to simulate, rad/sec
Nw = numel(w);
figure(2); clf;
for i=1:Nw
    % Evaluate TF at specified frequency
    Gw = evalfr(G,j*w(i));
    Gmag = abs(Gw);
    Gmag_db = 20*log10( Gmag );
    Gph_deg = (180/pi)*angle(Gw);
    
    % Plot circle on Bode plot
    figure(1);
    subplot(2,1,1);
    hold on;
    semilogx(w(i)/wn,Gmag_db,'ro');
    hold off;
    subplot(2,1,2);
    hold on;
    semilogx(w(i)/wn,Gph_deg,'ro');
    hold off;
    
    % Simulate system
    Tf = 50*pi;
    t = linspace(0,Tf,50*Tf);
    u = sin(w(i)*t);
    [y,t] = lsim(G,u,t);
    
    % Plot results
    figure(2);
    subplot(2,1,i);
    plot(t,u,'b-.',t,y,'r')
    xlabel('Time, t (sec)');
    ylabel('Output, y');
    grid on;
    set(gca, 'LooseInset', get(gca, 'TightInset'));
    
    % Set ticks
    xlim([Tf-4*pi/w(i) Tf]);
    XTick = Tf+[-4:1:0]*pi/w(i);
    XTickLabel = cell(1,numel(XTick));
    for k=1:numel(XTick)
        XTickLabel{k} =num2str(XTick(k),'%4.1f');
    end
    set(gca,'XTick',XTick,'XTickLabel',XTickLabel);
    if i==1
        legend('u, \omega=8','y','Location','SouthEast');
        set(gca,'Ytick',[-2 -1 0 1 2]);
    else
        legend('u, \omega=4','y','Location','SouthEast');
        set(gca,'Ytick',[-10 -5 0 5 10]);
    end
    set(gca,'LineWidth',1)
end

% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'SOResBode')
    
    figure(2)
    garyfyFigure
    myprint(2,'SOResResp')
end

