%% Sinusoidal Response: Principle of Superposition

%% Specify Signal
% The signal specified below is a sum of three terms. The constant term
% can be interepreted as a cosine at zero frequency.  Thus the three terms
% correspond to cosines at DC (constant), low, and high frequency.

% Time Vector
N = 1000;
T = 3;
t = linspace(0,T,N);
t = t(:);

% Specify amplitude, phase, and frequency for each component of signal.
A = [1.2; 0.9; 0.9];            % Amplitudes
thetaDeg = [0; 90; 30];           % Phase, degs
thetaRad = thetaDeg*pi/180;         % Phase, rads
w = [0; 10; 100];                % Frequency, rad/sec
Ncos = numel(A);                % Number of cosine terms in signal

% Form signal using frequencies, amplitudes, and phases given above.
u = zeros(N,1);
for i=1:Ncos
    u = u + A(i)*cos( w(i)*t + thetaRad(i) );
end

% Plot Signal vs. Time
figure(1)
clf
plot(t,u,'b');
xlabel('Time, sec');
ylabel('u');
grid on;

% Plot Signal Amplitude and Phase vs. Frequency
ww=[w w w]'; 
AA = [zeros(Ncos,1) A zeros(Ncos,1)]';
pp = [zeros(Ncos,1) thetaDeg zeros(Ncos,1)]';

figure(2)
clf
subplot(2,1,1);
plot(ww,AA,'b',w,A,'bo');
ylabel('Amplitude');
title('Frequency Content');
set(gca,'Ytick',unique(A));
set(gca,'Xtick',[0:10:110]);
grid on;

subplot(2,1,2);
plot(ww,pp,'b',w,thetaDeg,'bo');
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
set(gca,'Ytick',unique(thetaDeg))
set(gca,'Xtick',[0:10:110]);
grid on;

% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'uTimeDom')
    
    figure(2)
    garyfyFigure
    myprint(2,'uFreqDom')
end

%% System Response

% System
G = tf(10,[1 10]);

% Response due to input u with zero IC
[y,t] = lsim(G,u,t);

% Steady-state response 
yss = zeros(N,1);
yA = zeros(Ncos,1);
ythetaRad = zeros(Ncos,1);
ythetaDeg = zeros(Ncos,1);
for i=1:Ncos
    Gw = freqresp(G,w(i));
    
    yA(i) = abs(Gw)*A(i);
    ythetaRad(i) = thetaRad(i) + angle(Gw);
    ythetaDeg(i) = ythetaRad(i)*180/pi;
    
    yss = yss + yA(i)*cos( w(i)*t + ythetaRad(i) );
end

% Plot Response vs. Time
figure(3)
clf
plot(t,y,'r',t,yss,'b-.');
legend('Actual','Steady-State','Location','SouthEast');
xlabel('Time, sec');
ylabel('y');
grid on;


% Plot Output Amplitude and Phase vs. Frequency
AA = [zeros(Ncos,1) yA zeros(Ncos,1)]';
pp = [zeros(Ncos,1) ythetaDeg zeros(Ncos,1)]';

figure(4)
clf
subplot(2,1,1);
plot(ww,AA,'b',w,yA,'bo');
ylabel('Amplitude');
title('Frequency Content');
set(gca,'Ytick',sort(yA),'YtickLabel', num2str(sort(yA),'%4.2f'));
set(gca,'Xtick',[0:10:110]);
grid on;

subplot(2,1,2);
plot(ww,pp,'b',w,ythetaDeg,'bo');
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
set(gca,'Ytick',sort(ythetaDeg),'YtickLabel',num2str(sort(ythetaDeg),'%4.2f'));
set(gca,'Xtick',[0:10:110]);
ylim([-60 60]);
grid on;


% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(3)
    garyfyFigure
    myprint(3,'yTimeDom')
    
    figure(4)
    garyfyFigure
    myprint(4,'yFreqDom')
end


