function [A,theta,w] = cosDFT(y,t)
% This function uses the Fast Fourier Transform (FFT) algorithm to compute
% the Discrete Fourier Transform (DFT) of a sampled signal y(t). The 
% complex DFT amplitudes are converted into amplitudes/phases for a real
% cosine wave decomposition:
%    y(t) = sum_{k=1}^Nmax A(k)*cos( w(k)*t + theta(k) )
% where Nmax = floor( (N+1)/2 ). 
%
% Inputs:
%  y: N-by-1 vector of the sampled data.
%  t: N-by-1 vector of evenly spaced sample times.
%
% Outputs:
%  A: Nmax-by-1 vector of amplitudes. The first term A(1) corresponds to 
%     the amplitude of the constant (DC) component w(1)=0.
%  theta: Nmax-by-1 vector of phases (rads). The first entry is zero if the 
%     constant term is positive and +/-pi if the constant term is negative.
%  w: Nmax-by-1 vector of frequencies.
%
% 9/10/16 PJS Initial Coding

%% Turn data into column vectors
y = y(:);
t = t(:);

%% Number of points and sample time 
% Samples are at [0, dt, 2*dt, ..., (N-1)*dt]
N = numel(t);
dt = t(2)-t(1);
if any( abs(diff(t)-dt)> 1e-10*dt )
    error('Sample times must be evenly spaced.')
end

%% Compute Discrete Fourier Transform using Fast Fourier Transform (FFT)
Y = fft(y);

%% Convert (complex) FFT amplitude Y into real amplitude/phase of sine

% Indices corresponding to frequencies [0, 2*pi/T,..., (N-1)*pi/T]
idx = (1:(N+1)/2);
idx = idx(:);

% Frequencies
w = (2*pi/dt)*(idx-1)/N;
w = w(:);

% Amplitudes
A = 2*abs(Y(idx))/N;
A(1) = abs(Y(1))/N;

% Phase
theta = angle(Y(idx));
if A(1)>=0
    theta(1) = 0;
else
    if theta(2) < 0
        theta(1) = -pi;
    else
        theta(1) = +pi;
    end
end

